@@ -1,9 +1,11 @@
 import gc
 import sys
+import warnings
 import weakref
 
 import pytest
 
+from gi import PyGIWarning
 from gi.repository import GObject, Gio, Regress
 
 
@@ -170,6 +172,64 @@ def test_object_with_signal_callback():
     assert new_obj.props.int == 42
 
 
+with warnings.catch_warnings():
+    warnings.filterwarnings("ignore", ".*use __slots__.*")
+
+    class SlotObj(Regress.TestObj):
+        __slots__ = ["py_int"]
+
+        def __init__(self):
+            super().__init__()
+
+
+def test_class_with_slots_raises_warning():
+    with warnings.catch_warnings(record=True) as warn:
+
+        class _SlotClass(GObject.Object):
+            __slots__ = ["py_int"]
+
+    assert warn[0].category is PyGIWarning
+    assert "SlotClass shouldn't use __slots__." in str(warn[0].message)
+
+
+def test_subclass_with_slots_raises_warning():
+    class BaseClass(GObject.Object):
+        pass
+
+    with warnings.catch_warnings(record=True) as warn:
+
+        class _SlotClass(BaseClass):
+            __slots__ = ["py_int"]
+
+    assert warn[0].category is PyGIWarning
+    assert "SlotClass shouldn't use __slots__." in str(warn[0].message)
+
+
+def test_slot_object_can_be_created():
+    container = Regress.TestObj()
+    obj = SlotObj()
+    obj.py_int = 42
+    container.props.bare = obj
+
+    del obj
+    gc.collect()  # for pypy
+    new_obj = container.props.bare
+
+    assert new_obj.py_int == 42
+
+
+def test_slot_object_without_values_can_be_created():
+    container = Regress.TestObj()
+    obj = SlotObj()
+    container.props.bare = obj
+
+    del obj
+    gc.collect()  # for pypy
+    new_obj = container.props.bare
+
+    assert new_obj
+
+
 def test_objects_with_cyclic_dependency_and_instance_dict():
     # Test with a cycle:
     #
@@ -224,6 +284,31 @@ def test_objects_with_cyclic_dependency_without_instance_dict():
     assert new_d.props.bare is new_b
 
 
+def test_objects_with_cyclic_dependency_and_instance_dict_no_content():
+    # Test with a cycle:
+    #
+    # b <-> a <-> c
+    #       ^---> d
+    a, b, c, d = [Regress.TestObj(int=i) for i in range(4)]
+
+    b.name = "b"
+
+    a.props.bare = b
+    a.all = [b, c, d]
+    b.props.bare = a
+    b.a = a
+    c.props.bare = a
+    c.a = a
+    d.props.bare = a
+    d.a = a
+    del b, c, d
+
+    gc.collect()
+    gc.collect()
+
+    assert a.props.bare.name == "b"
+
+
 def test_chained_objects_are_collected():
     # a --> b --> c
     a, b, c = [Regress.TestObj(int=i) for i in range(3)]
@@ -282,12 +367,56 @@ def test_gobject_cycle_is_collected():
 
 def test_object_with_post_init():
     class PostInit(Regress.TestObj):
+        number = GObject.Property(
+            type=int, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT
+        )
+
         def do_constructed(self):
             self.post_init_called = True
+            assert self.number == 42
 
-    obj = PostInit()
+    obj = PostInit(number=42)
 
     assert obj.post_init_called
+    assert obj.number == 42
+
+
+def test_object_with_post_init_created_by_new():
+    class PostInit(Regress.TestObj):
+        number = GObject.Property(
+            type=int, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT
+        )
+
+        def do_constructed(self):
+            self.post_init_called = True
+            assert self.number == 42
+
+    obj = GObject.new(PostInit, number=42)
+
+    assert obj.post_init_called
+    assert obj.number == 42
+
+
+class InitializationOrder(Regress.TestObj):
+    def __init__(self):
+        self.order = ["__init__1"]
+        super().__init__()
+        self.order.append("__init__2")
+
+    def do_constructed(self):
+        self.order.append("do_constructed")
+
+
+def test_object_constructed_after_init():
+    obj = InitializationOrder()
+
+    assert obj.order == ["__init__1", "do_constructed", "__init__2"]
+
+
+def test_object_constructed_after_init_by_new():
+    obj = GObject.new(InitializationOrder)
+
+    assert obj.order == ["__init__1", "__init__2", "do_constructed"]
 
 
 def test_object_with_post_init_and_interface():
@@ -320,3 +449,5 @@ def test_object_with_post_init_raises_exception():
 
     with pytest.raises(ValueError, match="Catch me"):
         PostInit()
+    with pytest.raises(ValueError, match="Catch me"):
+        GObject.new(PostInit)
