@@ -115,7 +115,7 @@ class TestVFuncsWithObjectArg(unittest.TestCase):
 
         del vfuncs
         gc.collect()
-        self.assertTrue(vfuncs_ref() is None)
+        self.assertIsNone(vfuncs_ref())
 
     def test_vfunc_return_object_transfer_none(self):
         # This tests a problem case where the vfunc returns a GObject owned solely by Python
@@ -642,7 +642,6 @@ class TestVFuncsWithHeldFloatingArg(unittest.TestCase):
         self.assertTrue(held_object_ref() is None)
 
 
-@unittest.skipIf(Regress is None, "Regress is required")
 class TestArgumentTypeErrors(unittest.TestCase):
     def test_object_argument_type_error(self):
         # ensure TypeError is raised for things which are not GObjects
